import time

import PySide2
from PySide2.QtCore import QTimer, Qt
from PySide2.QtWidgets import QMainWindow

from .uimanager import UiManager
from .statemanager import StateManager
from .state import State
from .gamescreen import GameScreen


class MainWindow(QMainWindow):

    def __init__(self, app, state: State, screenFPS, fixedFPS, parent=None):
        """
        The MainWindow is created by the application, here we handle ui/scenes and send out updates.
        :param state: The current state we want to start with.
        :param parent: The widget this held under, None by default because this is top widget
        """
        QMainWindow.__init__(self, parent)
        self.app = app  # the application created
        self.gameScreen = None  # where all graphics are drawn
        self.uiManager = UiManager(self, customWidgets=[GameScreen])  # Loads widgets into window from a file
        self.stateManager = StateManager(self, state)  # Manages state updates and transitions

        self.updateFPSTimer = QTimer(self)  # Used to manage frame timings
        self.updateFPSTimer.timeout.connect(self.__calculateFPS)

        self.fixedFps = 0  # Calculated FPS
        self.screenFps = 0

        self.fixedFrames = 0  # Accumulated Frames
        self.screenFrames = 0

        self.targetFixedFPS = fixedFPS
        self.targetUpdateFPS = screenFPS
        self.fixedTiming = 1 / self.targetFixedFPS
        self.screenTiming = 1 / self.targetUpdateFPS
        self.fixedNeeded = fixedFPS
        self.screenNeeded = screenFPS
        self.lastTime = 0
        self.setMouseTracking(True)

        self.uiManager.keepWidgets = self.children()

    def start(self):
        """
        Called to start the window
        :return: None
        """
        self.stateManager.initializeState()  # start the state
        self.updateFPSTimer.start(100)

    def closeEvent(self, event: PySide2.QtGui.QCloseEvent):
        """
        Called when the window its closed
        :return: None
        """
        print("Window closed.")
        self.updateFPSTimer.stop()
        self.stateManager.exit()

    def __calculateFPS(self):
        """
        Averages FPS and adjust frame timings
        :return: None
        """

        self.fixedFps = self.fixedFrames / (time.perf_counter() - self.lastTime)
        self.fixedFrames = 0

        self.screenFps = self.screenFrames / (time.perf_counter() - self.lastTime)
        self.screenFrames = 0

        if -0.5 < (self.targetFixedFPS - self.fixedFps) / self.targetFixedFPS < 0.5 and self.fixedNeeded > 30:
            self.fixedNeeded += (self.targetFixedFPS - self.fixedFps) * 0.25
            self.fixedTiming = 1 / self.fixedNeeded

        if -0.5 < (self.screenNeeded - self.screenFps) / self.targetUpdateFPS < 0.5 and self.screenNeeded > 30:
            self.screenNeeded += (self.targetUpdateFPS - self.screenFps) * 0.15
            self.screenTiming = 1 / self.screenNeeded

        self.lastTime = time.perf_counter()

    def resizeEvent(self, event: PySide2.QtGui.QResizeEvent):
        if self.gameScreen:
            self.gameScreen.setGeometry(0, 0, self.width(), self.height())

    def keyPressEvent(self, event: PySide2.QtGui.QKeyEvent):
        self.stateManager.keyPressed(event)

    def keyReleaseEvent(self, event: PySide2.QtGui.QKeyEvent):
        self.stateManager.keyReleased(event)

    def mouseMoveEvent(self, event: PySide2.QtGui.QMouseEvent):
        self.stateManager.mouseMoved(event)

    def mousePressEvent(self, event: PySide2.QtGui.QMouseEvent):
        self.stateManager.mousePressed(event)

    def mouseReleaseEvent(self, event: PySide2.QtGui.QMouseEvent):
        self.stateManager.mouseReleased(event)

    def waitForLoad(self):
        QTimer(self).singleShot(0, self.stateManager.start)



from PySide2.QtCore import QRunnable, Slot, QThreadPool

from .state import State
from .gamescreen import GameScreen

import time


class Physics(QRunnable):

    def __init__(self, fixedUpdate, state):
        QRunnable.__init__(self)
        self.fixedUpdate = fixedUpdate
        self.window = state.window
        self.state = state
        self.setAutoDelete(False)

    @Slot()  # QtCore.Slot
    def run(self):
        while self.state.active:
            start = time.perf_counter()
            self.fixedUpdate()
            self.window.fixedFrames += 1
            wait = self.window.fixedTiming - (time.perf_counter() - start)
            time.sleep(wait if wait > 0 else 0)


class Rendering(QRunnable):

    def __init__(self, update, state):
        QRunnable.__init__(self)
        self.update = update
        self.window = state.window
        self.state = state
        self.setAutoDelete(False)

    @Slot()  # QtCore.Slot
    def run(self):
        while self.state.active:
            start = time.perf_counter()
            self.update()
            if self.window.gameScreen:
                self.window.gameScreen.update()
            self.window.screenFrames += 1
            wait = self.window.screenTiming - (time.perf_counter() - start)
            time.sleep(wait if wait > 0 else 0)


class StateManager:

    def __init__(self, window, state: State):
        self.window = window
        self.currentState = state
        self.threadPool = QThreadPool()

    def changeState(self, state: State):
        """
        Switch states
        :param state: The next state to show
        :return: None
        """

        self.currentState.active = False
        self.currentState.exitGOH()
        self.currentState.exit()
        self.currentState = state
        self.initializeState()

    def update(self):
        self.currentState.updateGOH()
        self.currentState.update()

    def fixedUpdate(self):
        self.currentState.fixedUpdateGOH()
        self.currentState.fixedUpdate()

    def initializeState(self):
        self.currentState.window = self.window  # Give the state a reference to the window
        self.currentState.loadUi()  # Load the states UI
        self.window.waitForLoad()

    def start(self):
        self.window.gameScreen = self.window.findChild(GameScreen)
        self.currentState.startGOH()
        self.currentState.start()  # Start the state
        self.threadPool.start(Physics(self.fixedUpdate, self.currentState))
        self.threadPool.start(Rendering(self.update, self.currentState))

    def exit(self):
        self.currentState.active = False
        self.currentState.exitGOH()
        self.currentState.exit()

    def keyPressed(self, event):
        self.currentState.keyPressed(event)
        self.currentState.gameObjectHandler.keyPressed(event)

    def keyReleased(self, event):
        self.currentState.keyReleased(event)
        self.currentState.gameObjectHandler.keyReleased(event)

    def mouseMoved(self, event):
        self.currentState.mouseMoved(event)
        self.currentState.gameObjectHandler.mouseMoved(event)

    def mousePressed(self, event):
        self.currentState.mousePressed(event)
        self.currentState.gameObjectHandler.mousePressed(event)

    def mouseReleased(self, event):
        self.currentState.mouseReleased(event)
        self.currentState.gameObjectHandler.mouseReleased(event)



import pytmx
import Box2D
import importlib
from PIL import ImageColor
from .gameobject import GameObject


class TileMap:

    def __init__(self, tmxFile, gameObjectHandler, gameScreen):
        tiled_map = pytmx.TiledMap(tmxFile, image_loader=gameScreen.image_loader)
        if tiled_map.background_color:
            color = ImageColor.getcolor(tiled_map.background_color, 'RGB')
            gameScreen.bgColor = [color[0] / 255, color[1] / 255, color[2] / 255]

        hitboxes = {}
        objects = tiled_map.objects

        # Looks for objects and gets its vertices
        for obj in objects:
            pairPoints = []
            for pair in obj.points:
                pairPoints.append(
                    (-(pair[0] - obj.x) / gameObjectHandler.scale, -(pair[1] - obj.y) / gameObjectHandler.scale))
            hitboxes[obj.name] = pairPoints

        for layer in tiled_map:
            # Checks for layers with tiles
            if isinstance(layer, pytmx.TiledTileLayer):
                for x, y, image in layer.tiles():
                    # Gets class name and gets class name definition
                    className = layer.properties.get('class', 'GameObject')

                    if className != 'GameObject':
                        className = getattr(importlib.import_module('__main__'), className)
                    else:
                        className = GameObject
                    # Sets up Texture
                    gm = className()
                    gm.textureID = image[0]

                    gm.setWidth(image[1] / gameObjectHandler.scale ** 2)
                    gm.setHeight(image[2] / gameObjectHandler.scale ** 2)

                    # Sets up Color
                    color = layer.properties.get('color', None)
                    if color:
                        color = ImageColor.getcolor(color, 'RGBA')
                        color = [color[2] / 255, color[1] / 255, color[0] / 255, color[3] / 255]
                        print(color)

                    # Sets up Hitbox
                    verticesName = layer.properties.get('hitboxName', None)

                    # Sets up Body Type
                    bodyType = layer.properties.get('bodyType', 'none')
                    if bodyType != 'none':
                        bodyDef = Box2D.b2BodyDef()
                        if bodyType == 'dynamic':
                            bodyDef.type = Box2D.b2_dynamicBody
                        elif bodyType == 'static':
                            bodyDef.type = Box2D.b2_staticBody
                        else:
                            bodyDef.type = Box2D.b2_kinematicBody
                        bodyDef.linearDamping = layer.properties.get('linearDamping', 0)
                        bodyDef.angularDamping = layer.properties.get('angularDamping', 0)
                        bodyDef.fixedRotation = layer.properties.get('fixedRotation', False)

                        # Linear Vel (Broken)
                        linearVel = layer.properties.get('linearVel', None)
                        if linearVel:
                            linearVel = [float(x) for x in linearVel.split(',')]
                            bodyDef.linearVelocity = (linearVel[0], linearVel[1])

                        # Sets up body Fixture
                        if verticesName:
                            bodyFixtureDef = Box2D.b2FixtureDef(
                                shape=Box2D.b2PolygonShape(vertices=hitboxes[verticesName]))
                        else:
                            bodyFixtureDef = Box2D.b2FixtureDef(shape=Box2D.b2PolygonShape(
                                box=((image[1] / gameObjectHandler.scale) / 2,
                                     (image[2] / gameObjectHandler.scale) / 2)))
                        bodyFixtureDef.density = layer.properties.get('density', 1)
                        bodyFixtureDef.friction = layer.properties.get('friction', 0)
                        bodyFixtureDef.restitution = layer.properties.get('restitution', 0)
                        # Sets up Position
                        bodyDef.position = (
                            -(x * tiled_map.tilewidth) / gameObjectHandler.scale,
                            -(y * tiled_map.tileheight) / gameObjectHandler.scale)
                        # adds box
                        if color:
                            gm.textureID = -1
                            gameObjectHandler.add(gm, bodyDef, bodyFixtureDef, color)
                        else:
                            gameObjectHandler.add(gm, bodyDef, bodyFixtureDef)
                    else:
                        gm.position = (-x / gameObjectHandler.scale, -y / gameObjectHandler.scale)
                        if color:
                            gm.textureID = -1
                            vertices = hitboxes[verticesName]
                            gm.vertices = [
                                (v[0] / gameObjectHandler.scale, v[1] / gameObjectHandler.scale) for v in vertices]
                            gameObjectHandler.add(gm, color=color)
                        else:
                            gameObjectHandler.add(gm)



from OpenGL.GL import *
import numpy
from PIL import Image
from PIL import ImageOps
from PySide2.QtWidgets import QOpenGLWidget


class GameScreen(QOpenGLWidget):

    def __init__(self, parent):
        """
        Subclass of the QOpenGLWidget, this is promoted in Qt Designer so that
        we can draw to the widget.
        :param parent: The widget that the this held under usually MainWindow
        """
        super(GameScreen, self).__init__(parent=parent)

        self.cameraPosition = [0, 0]
        self.cameraScale = 1
        self.renderInfoList = []
        self.bgColor = [0, 0, 0]

    def initializeGL(self):
        """
        Here we will override in order to setup OpenGL how we want
        :return: None
        """
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        glClearColor(self.bgColor[0], self.bgColor[1], self.bgColor[2], 1)
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glEnable(GL_TEXTURE_2D)

    def paintGL(self):
        """
        This is the function we'll override to draw to screen
        :return: None
        """

        glClearColor(self.bgColor[0], self.bgColor[1], self.bgColor[2], 1)
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glColor3f(1, 1, 1)
        glLoadIdentity()
        glScalef(self.cameraScale, self.cameraScale, 0)
        glTranslatef(-self.cameraPosition[0], -self.cameraPosition[1], 0)
        for renderInfo in self.renderInfoList:

            if renderInfo[0] == -1:
                glPushMatrix()
                glTranslatef(renderInfo[3][0], renderInfo[3][1], 0)
                glRotatef(renderInfo[4], 0, 0, 1)
                glColor4f(renderInfo[2][0], renderInfo[2][1], renderInfo[2][2], renderInfo[2][3])
                glPolygonMode(GL_FRONT, GL_FILL)
                glBegin(GL_POLYGON)
                for vertex in renderInfo[1]:
                    glVertex2f(vertex[0], vertex[1])
                glEnd()
                glPopMatrix()
            else:
                glPushMatrix()
                glTranslatef(renderInfo[3][0], renderInfo[3][1], 0)
                glRotatef(renderInfo[4], 0, 0, 1)
                glBindTexture(GL_TEXTURE_2D, renderInfo[0])
                glBegin(GL_QUADS)
                glTexCoord2f(0, 0)
                glVertex2f(renderInfo[1], renderInfo[2])
                glTexCoord2f(1, 0)
                glVertex2f(-renderInfo[1], renderInfo[2])
                glTexCoord2f(1, 1)
                glVertex2f(-renderInfo[1], -renderInfo[2])
                glTexCoord2f(0, 1)
                glVertex2f(renderInfo[1], -renderInfo[2])
                glEnd()
                glBindTexture(GL_TEXTURE_2D, 0)
                glPopMatrix()

    def resizeGL(self, w, h):
        glViewport(0, 0, w, h)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        aspect = w / h
        glOrtho(aspect, -aspect, -1.0, 1.0, 1.0, -1.0)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()

    def loadTexture(self, imageData, width, height):
        textureID = glGenTextures(1)
        glPixelStorei(GL_UNPACK_ALIGNMENT, 4)
        glBindTexture(GL_TEXTURE_2D, textureID)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, imageData)
        return textureID

    def loadImageTexture(self, fileName):
        image = Image.open(fileName)

        # Add alpha if the base image doesn't have one
        if not (fileName.split(".")[-1] == "png"):
            image.putalpha(256)

        # Making image data for OpenGL
        imageData = numpy.array(list(image.getdata()), numpy.uint8)
        return self.loadTexture(imageData, image.width, image.height)

    def image_loader(self, filename, colorkey, **kwargs):
        image = Image.open(filename)
        transparent = filename.split(".")[-1] == "png"

        def extract_image(rect=None, flags=None):
            # Cropping the image to the necessary size
            if rect:
                crop = image.crop((rect[0], rect[1], rect[0] + rect[2], rect[1] + rect[3]))
            else:
                crop = image

            if flags:
                # Handling the flags
                if flags.flipped_horizontally:
                    crop = ImageOps.mirror(crop)
                if flags.flipped_vertically:
                    crop = ImageOps.flip(crop)
            
            # Add alpha if the base image doesn't have one
            if not transparent:
                crop.putalpha(256)
            # Making image data for OpenGL
            imageData = numpy.array(list(crop.getdata()), numpy.uint8)
            return self.loadTexture(imageData, crop.width, crop.height), crop.width, crop.height

        return extract_image




from PySide2.QtWidgets import QApplication
from .mainwindow import MainWindow


class Application(QApplication):

    def __init__(self, state, screenFps=120, fixedFps=60, args=None):
        """
        Creates and starts the application.
        :param state: The initial state to launch the Application with
        :param args: System arguments passed in
        :param config: Set application parameters
        """
        if args:
            super(Application, self).__init__(args)
        else:
            super(Application, self).__init__()

        self.window = MainWindow(self, state, screenFps, fixedFps)

    def run(self):
        """
        Execute the application, this will start the state
        :return: None
        """
        self.window.start()
        self.window.show()
        self.exec_()



from AggiEngine.contactlistener import ContactListener
import math

from Box2D import *


class GameObjectHandler:

    def __init__(self, window, scale=16):
        self.timing = 1 / window.targetFixedFPS
        self.window = window
        
        self.world = Box2D.b2World(gravity=(0, -9.8))  # create instance of box 2d world
        self.world.contactListener = ContactListener()
        
        self.scale = scale  # scaling parameter, is equal to pixels to meter
        self.gameObjects = []  # game object list
        self.removeList = []

    def setScale(self, scale):
        self.scale = scale

    def update(self):
        for gameObject in self.gameObjects:
            if gameObject.active:
                gameObject.update()

    def fixedUpdate(self):
        self.world.Step(self.timing, 6, 2)
        self.world.ClearForces()

        newRenderInfoList = []
        for gameObject in self.gameObjects:
            info = [gameObject.textureID]

            if gameObject.textureID == -1:
                if len(gameObject.color) < 4:
                    gameObject.color.append(1)
                info += [gameObject.vertices, gameObject.color]
            else:
                info += [gameObject.width, gameObject.height]

            if gameObject.active:
                gameObject.fixedUpdate()
                if gameObject.body is not None:
                    gameObject.position = gameObject.body.position / self.scale
                    gameObject.rotation = gameObject.body.angle
                info += [gameObject.position, math.degrees(gameObject.rotation)]
            newRenderInfoList.append(info)
        if self.window.gameScreen is not None:
            self.window.gameScreen.renderInfoList = newRenderInfoList

        for gameObject in self.removeList:
            self.gameObjects.remove(gameObject)
            self.world.DestroyBody(gameObject.body)
        self.removeList = []

    def add(self, gameObject, bodyDef=None, bodyFixtureDef=None, color=None):
        self.gameObjects.append(gameObject)  # adds game object to list of game objects
        gameObject.gameObjectHandler = self

        if bodyDef:
            body = self.world.CreateBody(bodyDef)
            body.CreateFixture(bodyFixtureDef)
            gameObject.body = body
            gameObject.body.userData = gameObject

            gameObject.body.userData = gameObject

            if type(bodyFixtureDef.shape) is b2PolygonShape:
                gameObject.vertices.clear()
                for vertex in bodyFixtureDef.shape.vertices:
                    gameObject.vertices.append([vertex[0] / self.scale, vertex[1] / self.scale])
            elif type(bodyFixtureDef.shape) is b2CircleShape:
                vertices = []
                for i in range(0, 30):
                    rad = (2 * math.pi * i) / 30
                    r = bodyFixtureDef.shape.radius / self.scale
                    vertices.append([(r * math.cos(rad) - (bodyFixtureDef.shape.pos[0] / self.scale)),
                                     (r * math.sin(rad) - (bodyFixtureDef.shape.pos[1] / self.scale))])
                gameObject.vertices = vertices

            if color is None:
                gameObject.color = [1, 1, 1, 1]
            else:
                gameObject.color = color
            gameObject.position = body.position / self.scale

        gameObject.window = self.window
        gameObject.start()

    def getGameObject(self, typeOf):
        for gameObject in self.gameObjects:
            if isinstance(gameObject, typeOf):
                return gameObject

    def getGameObjects(self, typeOf):
        gameObjects = []
        for gameObject in self.gameObjects:
            if isinstance(gameObject, typeOf):
                gameObjects.append(gameObject)
        return gameObjects

    def removeGameObject(self, toRemove):
        self.removeList.append(toRemove)

    def removeGameObjects(self, typeOf):
        for gameObject in self.gameObjects:
            if isinstance(gameObject, typeOf):
                self.removeList.append(gameObject)

    def exit(self):
        for gameObject in self.gameObjects:
            gameObject.exit()

    def keyPressed(self, event):
        for gameObject in self.gameObjects:
            gameObject.keyPressed(event)

    def keyReleased(self, event):
        for gameObject in self.gameObjects:
            gameObject.keyReleased(event)

    def mouseMoved(self, event):
        for gameObject in self.gameObjects:
            gameObject.mouseMoved(event)

    def mousePressed(self, event):
        for gameObject in self.gameObjects:
            gameObject.mousePressed(event)

    def mouseReleased(self, event):
        for gameObject in self.gameObjects:
            gameObject.mouseReleased(event)



from PySide2.QtCore import QMetaObject
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QMainWindow


class UiManager(QUiLoader):

    def __init__(self, window, customWidgets=None):
        QUiLoader.__init__(self)
        self.window = window

        # Register all the custom widgets so that they can be created later
        self.customWidgets = customWidgets
        if customWidgets is not None:
            for wid in customWidgets:
                self.registerCustomWidget(wid)

        self.keepWidgets = []

    def createWidget(self, class_name, parent=None, name=''):
        """
        Overrides QUiLoader to createWidget in current window rather than a new one
        :param class_name: The class we want to create
        :param parent: The parent widget
        :param name: The name of the widget we'll create
        :return: The created widget
        """

        if class_name is QMainWindow.__name__:
            return self.window

        if parent is None and self.window:
            return self.window
        else:
            if class_name in self.availableWidgets():
                widget = QUiLoader.createWidget(self, class_name, parent, name)
                widget.show()
            else:
                try:
                    widget = self.customWidgets[class_name](parent)
                except (TypeError, KeyError) as e:
                    raise Exception(class_name, 'was not found are you sure it was promoted?')

            if self.window:
                setattr(self.window, name, widget)

            return widget

    def loadWidgets(self, ui_file, deleteCurrent=False):
        """
        Loads the current ui_file and if wanted deleted old widgets
        :param ui_file: The file path to load
        :param deleteCurrent: Remove old widgets
        :return: None
        """

        if len(self.window.children()) > 0 and deleteCurrent:
            for i in range(0, len(self.window.children())):
                if not(self.window.children()[i] in self.keepWidgets):
                    self.window.children()[i].deleteLater()

        widgets = self.load(ui_file)  # load widgets
        QMetaObject.connectSlotsByName(widgets)



from Box2D import b2ContactListener


class ContactListener(b2ContactListener):

    def __init__(self):
        b2ContactListener.__init__(self)

    def BeginContact(self, contact):
        bodyA = contact.fixtureA.body
        bodyB = contact.fixtureB.body
        bodyA.userData.beginContact(bodyB)
        bodyB.userData.beginContact(bodyA)

    def EndContact(self, contact):
        bodyA = contact.fixtureA.body
        bodyB = contact.fixtureB.body
        bodyA.userData.endContact(bodyB)
        bodyB.userData.endContact(bodyA)

    def PreSolve(self, contact, manifold):
        bodyA = contact.fixtureA.body
        bodyB = contact.fixtureB.body
        bodyA.userData.preSolve(bodyB, manifold)
        bodyB.userData.preSolve(bodyA, manifold)

    def PostSolve(self, contact, impulse):
        bodyA = contact.fixtureA.body
        bodyB = contact.fixtureB.body
        bodyA.userData.postSolve(bodyB, impulse)
        bodyB.userData.postSolve(bodyA, impulse)



from AggiEngine.gamescreen import GameScreen
from AggiEngine.state import State
from AggiEngine.statemanager import StateManager
from AggiEngine.uimanager import UiManager
from AggiEngine.gameobject import GameObject
from AggiEngine.mainwindow import MainWindow
from AggiEngine.particles import Particles
from AggiEngine.application import Application



class GameObject:

    def __init__(self, gameObjectHandler=None):
        self.body = None
        self.active = True
        self.window = None
        self.vertices = [[0, 0], [0.1, 0], [0.1, 0.1], [0, 0.1]]
        self.color = [1, 1, 1, 1]
        self.position = [0, 0]
        self.rotation = 0
        self.gameObjectHandler = gameObjectHandler
        self.textureID = -1
        self.height = 1
        self.width = 1

    def getWidth(self):
        return self.width * 2

    def getHeight(self):
        return self.height * 2

    def setWidth(self, width):
        self.width = width / 2

    def setHeight(self, height):
        self.height = height / 2

    def start(self):
        pass

    def update(self):
        pass

    def fixedUpdate(self):
        pass

    def exit(self):
        pass

    def keyPressed(self, event):
        pass

    def keyReleased(self, event):
        pass

    def mouseMoved(self, event):
        pass

    def mousePressed(self, event):
        pass

    def mouseReleased(self, event):
        pass

    def beginContact(self, body):
        pass

    def endContact(self, body):
        pass
    
    def preSolve(self, contact, manifold):
        pass

    def postSolve(self, contact, impulse):
        pass



from .gameobjecthandler import GameObjectHandler
from .tileloader import TileMap


class State:

    def __init__(self, ui_path=None, window=None):
        self.ui_path = ui_path
        self.window = window
        self.gameObjectHandler = None
        self.active = True

    def loadMap(self, filePath):
        TileMap(filePath, self.gameObjectHandler, self.window.gameScreen)

    def loadUi(self):
        if self.ui_path is not None:
            self.window.uiManager.loadWidgets(self.ui_path, True)

    def startGOH(self):
        self.gameObjectHandler = GameObjectHandler(self.window)

    def updateGOH(self):
        self.gameObjectHandler.update()

    def fixedUpdateGOH(self):
        self.gameObjectHandler.fixedUpdate()

    def exitGOH(self):
        self.gameObjectHandler.exit()

    def start(self):
        pass

    def update(self):
        pass

    def fixedUpdate(self):
        pass

    def exit(self):
        pass

    def keyPressed(self, event):
        pass

    def keyReleased(self, event):
        pass

    def mouseMoved(self, event):
        pass

    def mousePressed(self, event):
        pass

    def mouseReleased(self, event):
        pass



from distutils.core import setup
setup(
  name = 'AggiEngine',
  packages = ['AggiEngine'],
  version = '0.1',
  license='MIT',
  description = 'A game engine written in python',
  author = 'Brady Langdale, Nurivan Gomez',
  author_email = 'bradylangdale@tamu.edu, nurivan@tamu.edu',
  url = 'https://github.com/aggie-coding-club/AggiEngine/',
  download_url = 'https://github.com/aggie-coding-club/AggiEngine/archive/refs/tags/0.1.tar.gz',
  keywords = ['Python', 'Game Engine'],
  install_requires=[
        'PySide2',
        'PyOpenGL',
        'Box2D',
        'pytmx',
        'pillow',
        'numpy',
    ],
  classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)


import numpy as np

class Particles:

    def __init__(self, gameScreen, startColor=None, endColor=None, shape=None,
                 rate=0.5, count=100, endSize=0.01, sizeDecrease=0.95, colorFade=0.05):
        if not endColor:
            self.endColor = [1, 1, 1, 1]
        else:
            self.endColor = endColor

        if not startColor:
            self.startColor = [1, 1, 1, 1]
        else:
            self.startColor = startColor

        if not shape:
            self.shape = [[0, 0], [0.05, 0], [0.05, -0.05], [0, -0.05]]
        else:
            self.shape = shape

        self.gameScreen = gameScreen
        self.particles = []
        self.rate = rate
        self.count = count
        self.endSize = endSize
        self.sizeDecrease = sizeDecrease
        self.colorFade = colorFade
        self.time = 0

    def update(self):
        toRemove = []
        for particle in self.particles:
            if abs(particle[1][0][0] - particle[1][1][0]) < self.endSize:
                toRemove.append(particle)

            shape = []
            for vert in particle[1]:
                shape.append([vert[0] * self.sizeDecrease, vert[1] * self.sizeDecrease])
            particle[1] = shape
            particle[2] = self.getColor(particle[-1])
            particle[-1] -= self.colorFade
            self.gameScreen.renderInfoList.append(particle)

        for particle in toRemove:
            self.particles.remove(particle)

        self.time += self.rate

    def emit(self, position):
        if self.time > 1:
            if len(self.particles) < self.count:
                self.particles.append([-1, self.shape, self.getColor(0), position, 0, 0])
            self.time = 0

    def getColor(self, amount):
        r = abs(self.startColor[0] - self.endColor[0]) * amount + self.startColor[0]
        g = abs(self.startColor[1] - self.endColor[1]) * amount + self.startColor[1]
        b = abs(self.startColor[2] - self.endColor[2]) * amount + self.startColor[2]
        a = abs(self.startColor[3] - self.endColor[3]) * amount + self.startColor[3]
        return [max(r, self.endColor[0]), max(g, self.endColor[1]), max(b, self.endColor[2]), max(a, self.endColor[3])]